######################################################################
###################################################################### 
# Example of code analyzing the Quality of life dataset (Chen, West & Souza, 2006) 
# by using iterative Schmid-Leiman target rotation
#
# Abad, F.J., Garcia-Garzon, E., Garrido, L.E. & Barrada, J.R. (in press). 
# Iteration of Partially Specified Target Matrices: Application to the bi-factor case.
# Multivariate Behavioral Research.
#
# Please, first you need to install the packages psych and GPArotation 
#
# Furthermore, make sure that the file "Supporting_functions.R" is located 
# in the same working directory that "SLi.R" file.
#
# install.packages ("psych", dependencies = TRUE) 
# install.packages ("GPArotation, dependiences = TRUE)
######################################################################
######################################################################

######################################################################
## SLi Compute the Iterative Target Schmid-Leiman solution
#
# Correlation or covariance matrix are supported via fa function (psych package)
# Default estimation method is minres. Alternatives methods are detailed
# in fa function documentation (psych package)
# Rotation method for first-order factor analysis is Geomin. 

# Compute the iterative target factor rotation
#
# Args:
#
#   matrix: Correlation or Covariance matrix to be analyzed. Default is NULL.
#   specific_factors: number of specific factors to be extracted. Default is NULL.
#   fm: factor estimation method. Default is MINRES. Other alternatives can 
#       be found in the fa() function documentation (psych) package.
#   cutpoint: Value for cut-off point criterion (e.g., .20). Default is .20
#   iterations: iterations number. Default is 20.  If 0, Schmid-Leiman with target rotation (without iterations) is performed.
#  
# Returns:
#
#   Loadings: Rotated factor loading matrix
#  


######################################################################
library(GPArotation)
library(psych)

# Load the necessary functions for estimating SLi rotation
source("SLi_functions.R")

set.seed(999)
######################################################################

######################################################################
# Load Chen variance-covariance matrix (Chen, West & Souza, 2006) from psych package
# and convert into a correlation matrix
chen_cov  <- Chen [2:18, 2:18]
chen_cor  <- cov2cor(chen_cov)
round(chen_cor,3)
######################################################################


######################################################################
# Perform the Schmid-Leiman iterated rotation method 
# The number of specific factors is 4 as in Chen, West & Souza (2006)
# The number of iterations is fixed to 20
# The cut-off point for specifying the target matrix is .20

SLi_loadings <-   SLi (matrix = chen_cor,
                       specific_factors = 4, 
                       fm = "minres",
                       iterations = 20,
                       cutpoint = .20)

round(SLi_loadings$loadings,2)
######################################################################
